import os
from fastapi import FastAPI,Response, Depends
from fastapi.datastructures import Default, DefaultPlaceholder
from fastapi.responses import FileResponse,JSONResponse
from fastapi.staticfiles import StaticFiles
from fastapi.types import IncEx
from .router import Router
import sys
from pathlib import Path
from typing import Any, List, Dict, Sequence
from functools import wraps
from scalar_fastapi import get_scalar_api_reference


svg_data_uri ="""data:image/svg+xml;base64,<svg width='100' height='100' viewBox='0 0 100 100' fill='none' xmlns='http://www.w3.org/2000/svg'>
        <rect width='100' height='100' fill='#CBECE3'/>
        <path d='M27 78V22H30.1379L69.2414 60.0575V22H72.2184V78H27Z' fill='#1CB68D'/>
        </svg>
        """
def Nexy(title: str = None , favicon:str = svg_data_uri,**args):
    if title is None:
        title = Path.cwd().name 

    app:FastAPI = FastAPI(title=title,docs_url="/122xxxxxx2345",redoc_url="/xx123n134" , **args)
    
    @app.get("/docs",include_in_schema=False)
    async def scalar_html():

        
        return get_scalar_api_reference(
            openapi_url=app.openapi_url,
            title=app.title,
            scalar_favicon_url= favicon
        )
    directory = "public"
    if not os.path.exists(directory):
        app.mount("/public",StaticFiles(directory="public"), name="Public")
    app.include_router(Router())
    # Configurer le cache
    cache_dir = Path('./__pycache__/nexy')
    cache_dir.mkdir(exist_ok=True)
    sys.pycache_prefix = str(cache_dir)
    
    
    return app


# Définir le décorateur Params avec tous les paramètres fournis
def Params(
    *,
    response_model: Any = Default(None),
    status_code: int | None = None,
    tags: List[str ] | None = None,
    dependencies: Sequence[Depends] | None = None,
    summary: str | None = None,
    description: str | None = None,
    response_description: str = "Successful Response",
    responses: Dict[int | str, Dict[str, Any]] | None = None,
    deprecated: bool | None = None,
    operation_id: str | None = None,
    response_model_include: IncEx | None = None,
    response_model_exclude: IncEx | None = None,
    response_model_by_alias: bool = True,
    response_model_exclude_unset: bool = False,
    response_model_exclude_defaults: bool = False,
    response_model_exclude_none: bool = False,
    include_in_schema: bool = True,
    response_class: type[Response] | DefaultPlaceholder = Default(JSONResponse),
    name: str | None = None,
    openapi_extra: Dict[str, Any] | None = None,
):
    """
    Un décorateur pour ajouter des paramètres personnalisés à une route.
    """

    def decorator(func):
        # Ajouter tous les paramètres à un dictionnaire
        func.params = {
            "response_model": response_model,
            "status_code": status_code,
            "tags": tags,
            "dependencies": dependencies,
            "summary": summary,
            "description": description,
            "response_description": response_description,
            "responses": responses,
            "deprecated": deprecated,
            "operation_id": operation_id,
            "response_model_include": response_model_include,
            "response_model_exclude": response_model_exclude,
            "response_model_by_alias": response_model_by_alias,
            "response_model_exclude_unset": response_model_exclude_unset,
            "response_model_exclude_defaults": response_model_exclude_defaults,
            "response_model_exclude_none": response_model_exclude_none,
            "include_in_schema": include_in_schema,
            "response_class": response_class,
            "name": name,
            "openapi_extra": openapi_extra,
        }

        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return result

        return wrapper

    return decorator




import os
from fastapi import APIRouter
from .utils import deleteFistDotte, dynamicRoute,importModule,convertPathToModulePath

# 
def FIND_ROUTES(base_path):
    routes:list = []
    
    # Verify if the 'app' folder exists
    if os.path.exists(base_path) and os.path.isdir(base_path):
        # Explore the 'app' folder and its subfolders
        for root, dirs, files in os.walk(base_path):
            #supprimers des _folder
            dirs[:] = [d for d in dirs if not d.startswith("_")]

            route = {
                "pathname": f"{'/' if os.path.basename(root) == base_path else '/' +  deleteFistDotte(os.path.relpath(root, base_path).replace('\\','/'))}",
                "dirName": root
            }
            controller = os.path.join(root, 'controller.py')

            # Check for files and add to dictionary
            if os.path.exists(controller):
                route["module"] = convertPathToModulePath(f"{root}/controller")    

            routes.append(route)

    return routes




def Router():
    """
    Charge dynamiquement les routes à partir du répertoire 'app'.
    """
    app = APIRouter()
    routes = FIND_ROUTES(base_path="app")
    HTTP_METHODES: tuple = ["DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT"]

    for route in routes:
        pathname = dynamicRoute(route_in=route["pathname"])

        if "module" in route:
            try:
                # Tentative d'importation du module
                module = importModule(path=route["module"])
                
                for function_name in dir(module):
                    function = getattr(module, function_name)

                    # Vérifie que la fonction est callable et a des annotations
                    if callable(function) and hasattr(function, "__annotations__"):
                        params = getattr(function, "params", {})

                        # Ajout de la route pour chaque méthode HTTP
                        if function_name in HTTP_METHODES:
                            try:
                                app.add_api_route(
                                    path=pathname,
                                    endpoint=function,
                                    methods=[function_name],
                                    **{key: value for key, value in params.items() if key != "tags"},
                                    tags=params.get("tags") if params.get("tags") else [pathname]
                                )
                            except Exception as e:
                                # Capture les erreurs spécifiques à l'ajout de la route
                                print(f"Erreur lors de l'ajout de la route {pathname} pour la méthode {function_name}: {e}")
                                app.add_api_route(
                                    path=pathname,
                                    endpoint=lambda: {"error": f"Erreur lors de la méthode {function_name} pour la route {pathname}. {str(e)}"},
                                    methods=[function_name],
                                    status_code=500
                                )

                        # Ajout d'une route WebSocket si la méthode 'Socket' existe
                        elif function_name == "SOCKET":
                            try:
                                app.add_api_websocket_route(f"{pathname}/ws", function)
                            except Exception as e:
                                print(f"Erreur lors de l'ajout du WebSocket pour la route {pathname}: {e}")
                                app.add_api_websocket_route(
                                    f"{pathname}/ws",
                                    lambda: {"error": f"Erreur lors de la connexion WebSocket pour la route {pathname}. {str(e)}"}
                                )

            except Exception as e:
                # Capture les erreurs d'importation ou d'autres erreurs générales
                print(f"Erreur lors du chargement du module {route['module']}: {e}")
                app.add_api_route(
                    path=pathname,
                    endpoint=lambda: {"error": f"Erreur lors du chargement du module {route['module']}. {str(e)}"},
                    methods=["GET"],
                    status_code=500
                )
    return app